// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/Graph.hpp>

#include <memory>

const std::map<std::string, std::shared_ptr<Vertex>>& Graph::getVertices() const
{
    return vertices_;
}

void Graph::addEdge(const std::string& vertexId1, const std::string& vertexId2)
{
    auto v1 = findOrAddVertex(vertexId1);
    auto v2 = findOrAddVertex(vertexId2);
    v1->addNeighbor(v2);
    v2->addNeighbor(v1);
}

std::shared_ptr<Vertex> Graph::findOrAddVertex(const std::string& vertexId)
{
    auto iter = vertices_.find(vertexId);
    if (iter != vertices_.end())
    {
        return (*iter).second;
    }
    else
    {
        auto v = std::make_shared<Vertex>(vertexId);
        vertices_.insert({ vertexId, v });
        return v;
    }
}
